/**
 * Module Dependencies
 *
 * @ignore
 */
const path = require('path')
const logger = require('structure-logger')
const exphbs = require('express-handlebars')

const hbs = exphbs.create()

/**
 * Router Class
 *
 * @public
 * @class Router
 */
class Router {

  /**
   * Router constructor
   *
   * @public
   * @constructor
   * @param {Object} options - Options
   */
  constructor(options = {}) {
    this.options     = options

    this.middleware  = options.middleware || []
    this.routes      = options.routes || []
    this.routesList  = {}

  }

  generateMiddleware() {
    const server  = this.server

    logger.debug('Generating middleware')

    for(let i = 0, l = this.middleware.length; i < l; i++) {
      const item = this.middleware[i]
      const middlewareGroup = item.middlewareGroup

      if(middlewareGroup) {
        logger.debug('Middleware group', item)
        // Plugins middleware group
        if(item.plugins) {
          this.generatePluginRoutes()
          this.generateIndexRoute()
        }

      // Regular middlware
      } else if(typeof item === 'function') {

        if(this.options.baseUrl) {
          server.use(this.options.baseUrl, item)
        } else {
          server.use(item)
        }

      }
    }

  }

  /**
   * Generate routes
   *
   * @private
   */
  generatePluginRoutes() {
    const server  = this.server
    const version = `${process.env.API_VERSION}`

    for(let i = 0, l = this.routes.length; i < l; i++) {
      const route = this.routes[i]

      const routes    = route.routes
      const routeName = route.routeName
      let routesList = {}

      server.use(`/api/${version}/${routeName}`, routes)

      routes.stack.forEach(function(middleware) {
        const path = `/${routeName}${middleware.route.path}`
        const method = Object.keys(middleware.route.methods)[0].toUpperCase()

        if(routesList[path]) {
          routesList[path].push(method)
        } else {
          routesList[path] = new Array(method)
        }
      })

      this.routesList[routeName] = routesList

    }
  }

  generateIndexRoute() {
    const server  = this.server
    server.engine('handlebars', hbs.engine)
    server.set('view engine', 'handlebars')
    server.set('views', path.join(__dirname, '/views'))
    const version = `${process.env.API_VERSION}`
    const routes = this.routes

    const routeMap = {}

    for (const key of Object.keys(routes)) {

      const module = routes[key]
      const moduleName = routes[key].routeName

      for (const entry of module.routes.stack) {

        const route = {
          path: '/' + moduleName + entry.route.path,
          methods: Object.keys(entry.route.methods)
        }

        if (!routeMap[moduleName]) {
          routeMap[moduleName] = [route]
        } else {
          routeMap[moduleName].push(route)
        }

      }

    }

    server.get(`/api/${version}/`, (req, res) => {

      res.render('index', {version, routeMap})

    })

  }

  start(server) {
    this.server = server

    this.generateMiddleware()
  }

  use() {

    this.server.use.apply(this.server, arguments)

  }

}

module.exports = Router
